// $Id: c_writer.h,v 1.3 2014/03/31 15:52:10 david Exp $ -*- c++ -*-
#ifndef __COMPACT_SEMANTICS_CWRITER_H__
#define __COMPACT_SEMANTICS_CWRITER_H__

#include <string>
#include <iostream>
#include <cdk/symbol_table.h>
#include "targets/basic_ast_visitor.h"
#include "targets/symbol.h"

namespace compact {

  /**
   * Traverse the syntax tree and generate the corresponding C code.
   */
  class c_writer: public basic_ast_visitor {
    cdk::symbol_table<compact::symbol> &_symtab;

  public:
    c_writer(std::shared_ptr<cdk::compiler> compiler, cdk::symbol_table<compact::symbol> &symtab) :
        basic_ast_visitor(compiler), _symtab(symtab) {
    }

  public:
    ~c_writer() {
      os().flush();
    }

  public:
    void do_nil_node(cdk::nil_node * const node, int lvl) {
    }
    void do_data_node(cdk::data_node * const node, int lvl) {
    }
    void do_composite_node(cdk::composite_node * const node, int lvl) {
    }
    void do_sequence_node(cdk::sequence_node * const node, int lvl);

  public:
    void do_integer_node(cdk::integer_node * const node, int lvl);
    void do_double_node(cdk::double_node * const node, int lvl);
    void do_string_node(cdk::string_node * const node, int lvl);
    void do_identifier_node(cdk::identifier_node * const node, int lvl);

  protected:
    void processUnaryExpression(cdk::unary_expression_node * const node, int lvl, const char *tag);

  public:
    void do_neg_node(cdk::neg_node * const node, int lvl);

  protected:
    void do_binary_expression_node(cdk::binary_expression_node * const node, int lvl, const char *op);

  public:
    void do_add_node(cdk::add_node * const node, int lvl);
    void do_sub_node(cdk::sub_node * const node, int lvl);
    void do_mul_node(cdk::mul_node * const node, int lvl);
    void do_div_node(cdk::div_node * const node, int lvl);
    void do_mod_node(cdk::mod_node * const node, int lvl);
    void do_lt_node(cdk::lt_node * const node, int lvl);
    void do_le_node(cdk::le_node * const node, int lvl);
    void do_ge_node(cdk::ge_node * const node, int lvl);
    void do_gt_node(cdk::gt_node * const node, int lvl);
    void do_ne_node(cdk::ne_node * const node, int lvl);
    void do_eq_node(cdk::eq_node * const node, int lvl);

  public:
    void do_while_node(cdk::while_node * const node, int lvl);
    void do_if_node(cdk::if_node * const node, int lvl);
    void do_if_else_node(cdk::if_else_node * const node, int lvl);

  public:
    void do_lvalue_node(compact::lvalue_node * const node, int lvl);

  public:
    void do_program_node(compact::program_node * const node, int lvl);
    void do_print_node(compact::print_node * const node, int lvl);
    void do_read_node(compact::read_node * const node, int lvl);
    void do_assignment_node(compact::assignment_node * const node, int lvl);

  };

} // compact

#endif
